library ieee;
use ieee.std_logic_1164.all;

library wbgen2;

package wbgen2_pkg is

  
  component wbgen2_dpssram
    generic (
      g_data_width : natural;
      g_size       : natural;
      g_addr_width : natural;
      g_dual_clock : boolean;
      g_use_bwsel  : boolean);
    port (
      clk_a_i   : in  std_logic;
      clk_b_i   : in  std_logic;
      addr_a_i  : in  std_logic_vector(g_addr_width-1 downto 0);
      addr_b_i  : in  std_logic_vector(g_addr_width-1 downto 0);
      data_a_i  : in  std_logic_vector(g_data_width-1 downto 0);
      data_b_i  : in  std_logic_vector(g_data_width-1 downto 0);
      data_a_o  : out std_logic_vector(g_data_width-1 downto 0);
      data_b_o  : out std_logic_vector(g_data_width-1 downto 0);
      bwsel_a_i : in  std_logic_vector((g_data_width+7)/8-1 downto 0);
      bwsel_b_i : in  std_logic_vector((g_data_width+7)/8-1 downto 0);
      rd_a_i    : in  std_logic;
      rd_b_i    : in  std_logic;
      wr_a_i    : in  std_logic;
      wr_b_i    : in  std_logic);
  end component;

  component wbgen2_eic
    generic (
      g_num_interrupts : natural;
      g_irq00_mode     : std_logic_vector(1 downto 0);
      g_irq01_mode     : std_logic_vector(1 downto 0);
      g_irq02_mode     : std_logic_vector(1 downto 0);
      g_irq03_mode     : std_logic_vector(1 downto 0);
      g_irq04_mode     : std_logic_vector(1 downto 0);
      g_irq05_mode     : std_logic_vector(1 downto 0);
      g_irq06_mode     : std_logic_vector(1 downto 0);
      g_irq07_mode     : std_logic_vector(1 downto 0);
      g_irq08_mode     : std_logic_vector(1 downto 0);
      g_irq09_mode     : std_logic_vector(1 downto 0);
      g_irq0a_mode     : std_logic_vector(1 downto 0);
      g_irq0b_mode     : std_logic_vector(1 downto 0);
      g_irq0c_mode     : std_logic_vector(1 downto 0);
      g_irq0d_mode     : std_logic_vector(1 downto 0);
      g_irq0e_mode     : std_logic_vector(1 downto 0);
      g_irq0f_mode     : std_logic_vector(1 downto 0);
      g_irq10_mode     : std_logic_vector(1 downto 0);
      g_irq11_mode     : std_logic_vector(1 downto 0);
      g_irq12_mode     : std_logic_vector(1 downto 0);
      g_irq13_mode     : std_logic_vector(1 downto 0);
      g_irq14_mode     : std_logic_vector(1 downto 0);
      g_irq15_mode     : std_logic_vector(1 downto 0);
      g_irq16_mode     : std_logic_vector(1 downto 0);
      g_irq17_mode     : std_logic_vector(1 downto 0);
      g_irq18_mode     : std_logic_vector(1 downto 0);
      g_irq19_mode     : std_logic_vector(1 downto 0);
      g_irq1a_mode     : std_logic_vector(1 downto 0);
      g_irq1b_mode     : std_logic_vector(1 downto 0);
      g_irq1c_mode     : std_logic_vector(1 downto 0);
      g_irq1d_mode     : std_logic_vector(1 downto 0);
      g_irq1e_mode     : std_logic_vector(1 downto 0);
      g_irq1f_mode     : std_logic_vector(1 downto 0));
    port (
      rst_n_i          : in  std_logic;
      clk_i            : in  std_logic;
      irq_i            : in  std_logic_vector(g_num_interrupts-1 downto 0);
      reg_imr_o        : out std_logic_vector(g_num_interrupts-1 downto 0);
      reg_ier_i        : in  std_logic_vector(g_num_interrupts-1 downto 0);
      reg_ier_wr_stb_i : in  std_logic;
      reg_idr_i        : in  std_logic_vector(g_num_interrupts-1 downto 0);
      reg_idr_wr_stb_i : in  std_logic;
      reg_isr_o        : out std_logic_vector(g_num_interrupts-1 downto 0);
      reg_isr_i        : in  std_logic_vector(g_num_interrupts-1 downto 0);
      reg_isr_wr_stb_i : in  std_logic;
      wb_irq_o         : out std_logic);
  end component;
  
end wbgen2_pkg;
